/***********************************************************************************************************************
**
** Copyright (C) 2021 BaseALT Ltd.
** Authors: Valery Sinelnikov <greh@basealt.ru>, Ivan Savin <svn17@basealt.ru>
**
** This program is free software; you can redistribute it and/or
** modify it under the terms of the GNU General Public License
** as published by the Free Software Foundation; either version 2
** of the License, or (at your option) any later version.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software
** Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
**
***********************************************************************************************************************/
#include "accesswoobus.h"
#include <curl/curl.h>

struct request_data {
    gint size;
    gchar* data;
};

gint write_data(void *ptr, gint size, gint nmemb, struct request_data *data);

gchar *request_to_woobus(gchar* push_data) {
    CURL *curl;
    struct request_data data;
    data.size = 0;
    data.data = malloc(4096); /* reasonable size initial buffer */
    if(0 == data.data) {
        g_critical("request_to_woobus()\nFailed to allocate memory.\n");
        return 0;
    }
    data.data[0] = '\0';
    CURLcode res;
    curl = curl_easy_init();

    if (curl) {
        curl_easy_setopt(curl, CURLOPT_UNIX_SOCKET_PATH, SOCKET_WOO);
        curl_easy_setopt(curl, CURLOPT_URL, "http://localhost/");
        curl_easy_setopt(curl, CURLMOPT_PUSHDATA, push_data);
        curl_easy_setopt(curl, CURLOPT_WRITEFUNCTION, write_data);
        curl_easy_setopt(curl, CURLOPT_WRITEDATA, &data);
        res = curl_easy_perform(curl);
        if(res != CURLE_OK) {
            g_critical("curl_easy_perform() failed: %s\n",
                       curl_easy_strerror(res));
        }
        curl_easy_cleanup(curl);
    }
    return data.data;
}

gint write_data(void *ptr, gint size, gint nmemb, struct request_data *data) {
    gint index = data->size;
    gint n = (size * nmemb);
    gchar* tmp;
    data->size += (size * nmemb);
    tmp = realloc(data->data, (guint)data->size + 1); /* +1 for '\0' */
    if(tmp) {
        data->data = tmp;
    } else {
        if(data->data) {
            free(data->data);
        }
        g_critical("write_data()\nFailed to allocate memory.\n");
        return 0;
    }
    memcpy((data->data + index), ptr, (guint)n);
    data->data[data->size] = '\0';

    return size * nmemb;
}

