/***********************************************************************************************************************
**
** Copyright (C) 2021 BaseALT Ltd.
** Authors: Valery Sinelnikov <greh@basealt.ru>, Ivan Savin <svn17@basealt.ru>
**
** This program is free software; you can redistribute it and/or
** modify it under the terms of the GNU General Public License
** as published by the Free Software Foundation; either version 2
** of the License, or (at your option) any later version.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software
** Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
**
***********************************************************************************************************************/
#include "alteratorgdbus.h"

/*!
 * \brief The _AlteratorGdbus struct Class fields
 */
struct _AlteratorGdbus {
    GObjectClass parent_class;
    GList *modules;
    AlteratorService* service;
};
G_DEFINE_TYPE (AlteratorGdbus, alterator_gdbus, G_TYPE_OBJECT)

static void
alterator_gdbus_class_init(AlteratorGdbusClass* self){
}

static void
alterator_gdbus_init(AlteratorGdbus* self){
    self->modules = NULL;
}

/*
 * Finding introspection files for an alterator
 * Creating an AlteratorModule object based on each found file
 * self Pointer to the object
 * dirname Path for search
 * keyword Criteria for search
 */
static void
alterator_gdbus_load_modules(AlteratorGdbus* self , gchar* dirname, gchar* keyword){
    GDir *dir = g_dir_open(dirname, 0, 0);

    const gchar *filename;

    while ((filename = g_dir_read_name(dir))) {
        GRegex *regex;
        GMatchInfo *match_info;
        regex = g_regex_new (keyword, (GRegexCompileFlags)0, (GRegexMatchFlags)0, NULL);
        g_regex_match (regex, filename, (GRegexMatchFlags)0, &match_info);
        if (g_match_info_matches (match_info))
        {
            gchar *word = g_match_info_fetch (match_info, 0);
            g_free (word);
            g_match_info_next (match_info, NULL);
            GString* path1 =g_string_new("");
            GString* filenameGString =g_string_new(filename);
            g_string_printf(path1, "%s%s",PATH_TO_INTROSPECTION ,filename);
            AlteratorModule* module = alterator_module_new(
                        g_string_free(path1, FALSE), g_string_free(filenameGString, FALSE));
            if(alterator_module_state(module)){
                self->modules = g_list_append(self->modules, module);
            }
        }
        g_match_info_free (match_info);
        g_regex_unref (regex);
    }
    g_dir_close(dir);
    g_info("alterator_gdbus_load_modules()");
}

/*!
 * \brief alterator_gdbus_lookup_module Finding the module for the caller
 * \param self Pointer to the object
 * \param object_path Criteria for search
 * \return Caller module
 */
AlteratorModule*
alterator_gdbus_lookup_module(AlteratorGdbus* self, const gchar* object_path)
{
    for(guint i = 0; i<g_list_length (self->modules); i++){
        AlteratorModule* module = g_list_nth(self->modules,i)->data;
        gchar* objectnames = alterator_module_get_objectname(module);
        if(!g_strcmp0(object_path, objectnames)) return  module;
    }
    g_info("alterator_gdbus_lookup_module()");
    return NULL;
}

/*!
 * \brief alterator_gdbus_new Creates an object with available modules
 *  Finding introspection files for an alterator
 *  Creating an AlteratorModule object based on each found file
 * \param dirname Path for search
 * \param keyword Criteria for search
 * \return
 */
AlteratorGdbus*
alterator_gdbus_new(gchar* dirname, gchar* keyword)
{
    AlteratorGdbus* self = ALTERATOR_GDBUS(g_object_new(ALTERATOR_TYPE_GDBUS, NULL));
    alterator_gdbus_load_modules(self, dirname, keyword);
    self->service = alterator_service_new();
    g_info("alterator_gdbus_new()");
    return self;
}

/*!
 * \brief alterator_gdbus_get_modules Returns a list with modules
 * \param self Pointer to the object
 * \return GList* with modules
 */
GList*
alterator_gdbus_get_modules(AlteratorGdbus* self)
{
    g_info("alterator_gdbus_get_modules()");
    return  self->modules;
}

AlteratorService*
alterator_gdbus_get_service(AlteratorGdbus* self){
    return self->service;
}
