/***********************************************************************************************************************
**
** Copyright (C) 2021 BaseALT Ltd.
** Authors: Valery Sinelnikov <greh@basealt.ru>, Ivan Savin <svn17@basealt.ru>
**
** This program is free software; you can redistribute it and/or
** modify it under the terms of the GNU General Public License
** as published by the Free Software Foundation; either version 2
** of the License, or (at your option) any later version.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software
** Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
**
***********************************************************************************************************************/
#include "alteratorgdbus.h"
#include "alteratormodule.h"
#include "accesswoobus.h"
#include "requesttowoobus.h"
#include "makeresponsetodbus.h"
#include "polkitcheck.h"
#include "logalteratordbus.h"
#include "servicealterator.h"

static GMainLoop *loop = NULL;
static GHashTable *properties = NULL;
#define WOO_ERROR "(woo-error"
#define SERVICE_PATH "/ru/basealt/alterator/service"
gchar* domainalt = "altdb";
#define G_LOG_DOMAIN    ((gchar*) domainalt)

/**
 *  Service call processing function
 */
static void handle_method_for_modules (GDBusConnection       *connection,
                                const gchar           *sender,
                                const gchar           *object_path,
                                const gchar           *interface_name,
                                const gchar           *method_name,
                                GVariant              *parameters,
                                GDBusMethodInvocation *invocation,
                                gpointer               user_data)
{
    GString *action = g_string_new(object_path+1);
    g_string_append(action, ".");
    g_string_append(action, method_name);
    gchar* actionId = g_string_free(action, FALSE);
    g_debug("handle_method_call()\nactionId  : %s\n", actionId);

    if (authorizationcheck(connection, sender, actionId, object_path)){
        AlteratorGdbus* bus = (AlteratorGdbus*)user_data;
        AlteratorModule* module = alterator_gdbus_lookup_module(bus, object_path);
        GDBusInterfaceInfo* iface = alterator_module_get_introspection_data(module)->interfaces[0];
        GDBusMethodInfo* method = g_dbus_interface_info_lookup_method (iface, method_name);
        gchar* request =  make_request_woobus(alterator_module_get_modulename(module), method, parameters);
        if (request == NULL){
            g_warning("woo-error backend not found");
            g_dbus_method_invocation_return_dbus_error (invocation,
                                                        "ru.basealt.alterator",
                                                        "Not correct method");
        } else {

        gchar* data = request_to_woobus(request);

        GString * wooerror_equal = g_string_new_len(WOO_ERROR, 10);
        GString * wooerror = g_string_new_len(data, 10);
        if(!g_string_equal(wooerror_equal, wooerror)){

            GList* answer_strings = NULL;
            GVariant* answer = NULL;
            answer_strings = converting_answer_to_strings(data, answer_strings,
                                                          alterator_module_get_modulename(module),
                                                          method);

            answer = make_response_for_dbus(answer_strings, method);
            g_dbus_method_invocation_return_value(invocation, answer);
        } else {
            g_dbus_method_invocation_return_dbus_error (invocation,
                                                        "ru.basealt.alterator",
                                                        data);
        }

        g_string_free (wooerror_equal,TRUE);
        g_string_free (wooerror,TRUE);
        free(data);
        }
        g_free(request);
        g_free(actionId);
    }
    else{
        g_dbus_method_invocation_return_dbus_error (invocation,
                                                    "ru.basealt.alterator",
                                                    "Not authorization check");
    }
}

static void handle_method_for_service (GDBusConnection      *connection,
                                const gchar                 *sender,
                                const gchar                 *object_path,
                                const gchar                 *interface_name,
                                const gchar                 *method_name,
                                GVariant                    *parameters,
                                GDBusMethodInvocation       *invocation,
                                gpointer                     user_data)
{
        if(g_strcmp0(method_name, "set_log_priority") == 0){
            gint priority;
            g_variant_get (parameters, "(i)", &priority);
                if(priority <= 0){
                    set_log_priority(0);
                }
                else if(priority >= 6){
                    set_log_priority(6);
                }
                else {
                    set_log_priority((guint)priority);
                }
        g_info("set_log_priority(%d)", get_log_priority());
        }
        if (g_strcmp0 (method_name, "get_log_priority") == 0){
            g_dbus_method_invocation_return_value (invocation,
                                                   g_variant_new ("(i)", get_log_priority()));
        g_info("set_log_priority(%d)", get_log_priority());
        }
}

static void handle_method_call (GDBusConnection       *connection,
                                const gchar           *sender,
                                const gchar           *object_path,
                                const gchar           *interface_name,
                                const gchar           *method_name,
                                GVariant              *parameters,
                                GDBusMethodInvocation *invocation,
                                gpointer               user_data)
{
    if(g_strcmp0(object_path, SERVICE_PATH)){
    handle_method_for_modules (connection,
                               sender,
                               object_path,
                               interface_name,
                               method_name,
                               parameters,
                               invocation,
                               user_data);

    }
    else{
    handle_method_for_service (connection,
                               sender,
                               object_path,
                               interface_name,
                               method_name,
                               parameters,
                               invocation,
                               user_data);
    }

}

static const GDBusInterfaceVTable interface_vtable =
{
    handle_method_call,
};

static void on_bus_acquired (GDBusConnection *connection,
                             const gchar     *name,
                             gpointer         user_data)
{
    guint id;
    AlteratorGdbus* bus = (AlteratorGdbus*)user_data;
    GError **error = NULL;
    GList *modulesnew = alterator_gdbus_get_modules(bus);
    for(guint i = 0;i < g_list_length (modulesnew); i++){
        id = g_dbus_connection_register_object
                (connection,
                 alterator_module_get_objectname(g_list_nth(modulesnew,i)->data),
                 alterator_module_get_introspection_data(g_list_nth(modulesnew,i)->data)->interfaces[0],
                &interface_vtable,
                user_data,
                NULL,
                error);
        g_assert (id > 0);
    }
    id = g_dbus_connection_register_object
            (connection,
            SERVICE_PATH,
            alterator_service_get_introspection_data(alterator_gdbus_get_service(bus))->interfaces[0],
            &interface_vtable,
            user_data,
            NULL,
            error);
    g_assert (id > 0);

}

//static void on_name_acquired (GDBusConnection *connection,
//                              const gchar     *name,
//                              gpointer         user_data)
//{
//}

//static void on_name_lost (GDBusConnection *connection,
//                          const gchar     *name,
//                          gpointer         user_data)
//{
//}

static void help () {
    g_printf("\n");
    g_printf("alterator-dbus -p <number>   Set logging level (1 - 6, default - 2).\n");
    g_printf("\n");
}

int main (int argc, char *argv[])
{

    if (argc == 3 && !g_strcmp0(argv[1], "-p") && (0 < (*argv[2] - '0') && (*argv[2] - '0') <= 6)){
        set_log_priority((guint)(*argv[2] - '0'));
    } else if (argc == 2 && (!g_strcmp0(argv[1], "-h") || !g_strcmp0(argv[1], "--help"))){
        help();
        return 0;
    } else {
        set_log_priority(2);
    }

    g_log_set_handler (domainalt, (GLogLevelFlags)(G_LOG_LEVEL_MASK | G_LOG_FLAG_FATAL
                       | G_LOG_FLAG_RECURSION), log_alterator_gdbus_cb, NULL);

    guint owner_id;
    AlteratorGdbus* bus = alterator_gdbus_new(PATH_TO_INTROSPECTION, KEY_WORD);
    owner_id = g_bus_own_name (G_BUS_TYPE_SYSTEM,
                               "ru.basealt.alterator",
                               G_BUS_NAME_OWNER_FLAGS_NONE,
                               on_bus_acquired,
                               /*on_name_acquired*/NULL,
                               /*on_name_lost*/NULL,
                               bus,
                               NULL);

    loop = g_main_loop_new (NULL, FALSE);
    g_main_loop_run (loop);
    g_bus_unown_name (owner_id);
    g_hash_table_unref (properties);
    return 0;
}
