/***********************************************************************************************************************
**
** Copyright (C) 2021 BaseALT Ltd.
** Authors: Valery Sinelnikov <greh@basealt.ru>, Ivan Savin <svn17@basealt.ru>
**
** This program is free software; you can redistribute it and/or
** modify it under the terms of the GNU General Public License
** as published by the Free Software Foundation; either version 2
** of the License, or (at your option) any later version.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software
** Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
**
***********************************************************************************************************************/
#include "makeresponsetodbus.h"

static gchar*
trim_prefix (gchar* variable_name) {
    for (gint i = 0; i < 50 && *variable_name != '#'; i++, variable_name++);//50 is insurance
    variable_name++; //skipping '#'
    g_info("trim_prefix()");
    return variable_name;
}

GVariant*
make_response_for_dbus (GList* list, GDBusMethodInfo* method) {
    GList* answer_gvariant = NULL;
    GDBusArgInfo **out_args = method->out_args;
    GVariantBuilder *builder = NULL;
    while (*out_args) {
        GRegex *regex;
        GString *resGString = g_string_new("");
        gchar* variable_name;
        GError *error = NULL;
        GMatchInfo *matchInfo_quotes;
        GRegex *regex_quotes;

        //process the prefix for checklistbox
        if (g_str_has_prefix((*out_args)->name, CHECKLIST_PREFIX)) {
            variable_name = (*out_args)->name;
            variable_name = trim_prefix (variable_name);
            g_debug("make_response_for_dbus()\nvariable name :%s \n", variable_name);
        } else {
            variable_name = (*out_args)->name;
        }

        //formation of regular expressions
        if (*(out_args+1)) {
            gchar* next_variable_name = (*(out_args+1))->name;

            if (g_str_has_prefix(next_variable_name, CHECKLIST_PREFIX)) {
                next_variable_name = trim_prefix (next_variable_name);
                g_debug("next_variable_name name :%s \n", next_variable_name);
            }
            g_string_append_printf(resGString, VARIABLE_VALUE, variable_name, next_variable_name);
        } else {
            g_string_append_printf(resGString, LAST_VARIABLE_VALUE, variable_name);
        }

        regex = g_regex_new (resGString->str, (GRegexCompileFlags)0, (GRegexMatchFlags)0, &error);
        regex_quotes = g_regex_new (REMOVE_QUOTES, (GRegexCompileFlags)0, (GRegexMatchFlags)0, &error);

        //parsing and forming a response
        if (!g_strcmp0((*out_args)->signature, "as") &&
                g_str_has_prefix((*out_args)->name, CHECKLIST_PREFIX)) {

            builder = g_variant_builder_new (G_VARIANT_TYPE ("as"));
            GMatchInfo *matchInfo = NULL;

            //take only the first line
            if (list != NULL) {
                g_regex_match (regex, g_list_nth(list,0)->data, (GRegexMatchFlags)0, &matchInfo);

                if (g_match_info_matches (matchInfo)) {
                    gchar *result = g_match_info_fetch (matchInfo, 0);
                    g_regex_match (regex_quotes, result, (GRegexMatchFlags)0, &matchInfo_quotes);
                    gchar *result_quotes = g_match_info_fetch (matchInfo_quotes, 0);

                    if (result_quotes != NULL && g_strcmp0(result_quotes, "")) {
                        gchar **tokens = g_strsplit(result_quotes, ";", -1);
                        gchar **elements = tokens;

                        while (*tokens) {
                            g_variant_builder_add (builder, "s", *tokens);
                            tokens++;
                        }

                        g_strfreev(elements);
                    }
                    g_free(result);
                    g_free(result_quotes);
                }
            }

            GVariant *value = g_variant_new ("as", builder);
            g_variant_builder_unref (builder);
            answer_gvariant = g_list_append(answer_gvariant, value);

        } else if (!g_strcmp0((*out_args)->signature, "as")) {
            builder = g_variant_builder_new (G_VARIANT_TYPE ("as"));

            for (GList *l = list; l != NULL; l = l->next) {
                GMatchInfo *matchInfo;

                g_regex_match (regex, l->data, (GRegexMatchFlags)0, &matchInfo);

                if (g_match_info_matches (matchInfo)) {
                    gchar *result = g_match_info_fetch (matchInfo, 0);
                    g_debug("%s :%s \n", variable_name, result);
                    g_regex_match (regex_quotes, result, (GRegexMatchFlags)0, &matchInfo_quotes);
                    gchar *result_quotes = g_match_info_fetch (matchInfo_quotes, 0);
                    g_variant_builder_add (builder, "s", result_quotes);
                    g_free(result);
                    g_free(result_quotes);
                }
            }
            GVariant *value = g_variant_new ("as", builder);
            g_variant_builder_unref (builder);
            answer_gvariant = g_list_append(answer_gvariant, value);
        } else if (!g_strcmp0((*out_args)->signature, "s")) {
            if (list != NULL) {
                GMatchInfo *matchInfo;
                GVariant *value = NULL;
                //take only the first line
                g_regex_match (regex, g_list_nth(list,0)->data, (GRegexMatchFlags)0, &matchInfo);

                if (g_match_info_matches (matchInfo)) {
                    gchar *result = g_match_info_fetch (matchInfo, 0);
                    g_debug("%s :%s \n", variable_name, result);
                    g_regex_match (regex_quotes, result, (GRegexMatchFlags)0, &matchInfo_quotes);
                    gchar *result_quotes = g_match_info_fetch (matchInfo_quotes, 0);

                    if (result_quotes == NULL) {
                        g_warning("makeresponsetodbus.c: make_response_for_dbus(): invalid response string.");
                        value = g_variant_new ((*out_args)->signature, "");
                    } else {
                        value = g_variant_new ((*out_args)->signature, result_quotes);
                    }

                    answer_gvariant = g_list_append(answer_gvariant, value);
                    g_free(result);
                    g_free(result_quotes);
                }
            } else {
                g_warning("makeresponsetodbus.c: make_response_for_dbus(): GList* list == NULL");
                GVariant *value = g_variant_new ((*out_args)->signature, "");
                answer_gvariant = g_list_append(answer_gvariant, value);
            }

        } else if (!g_strcmp0((*out_args)->signature, "b")) {
            if (list != NULL) {
                GMatchInfo *matchInfo;
                //take only the first line
                g_regex_match (regex, g_list_nth(list,0)->data, (GRegexMatchFlags)0, &matchInfo);

                if (g_match_info_matches (matchInfo)) {
                    gchar *result = g_match_info_fetch (matchInfo, 0);
                     g_debug ("%s :%s \n", variable_name, result);
                    GVariant *value = NULL;
                    if (result[1]== 't') {
                        g_debug("true:%s \n", variable_name);
                        value = g_variant_new ((*out_args)->signature, TRUE);
                    } else {
                        g_debug("false:%s \n", variable_name);
                        value = g_variant_new ((*out_args)->signature, FALSE);
                    }
                    answer_gvariant = g_list_append(answer_gvariant, value);
                    g_free(result);
                }
            } else {
                g_warning("makeresponsetodbus.c: make_response_for_dbus(): GList* list == NULL");
                GVariant *value = g_variant_new ((*out_args)->signature, FALSE);
                answer_gvariant = g_list_append(answer_gvariant, value);
            }

        } else {
            g_warning("make_response_for_dbus() make_response_for_dbus: unknown argument type.");
        }
        g_string_free(resGString, TRUE);
        out_args++;
    }

    /*
     * Create an array from a list.
     */
    GVariant *arr[g_list_length(answer_gvariant)];
    for (guint i = 0;i < g_list_length (answer_gvariant); i++) {
        arr[i] = (GVariant*)g_list_nth(answer_gvariant,i)->data;
    }

    GVariant *result = g_variant_new_tuple(arr, g_list_length (answer_gvariant));
    g_info("make_response_for_dbus()");
    return result;
}

GList*
converting_answer_to_strings (gchar* request, GList* list, const gchar* object_path, const GDBusMethodInfo* method) {
    GError *error = NULL;
    GMatchInfo *matchInfo;
    GRegex *regex;
    g_debug("converting_answer_to_strings() \nparsing_answer_start request =  %s\n", request);

    /**
    * CONTENTS_OF_PARENTHESES takes out the pieces
    * in parentheses from the text
    */
    gchar methodNameCopy[strlen(method->name)];
    GString *resGString = g_string_new("");
    strcpy(methodNameCopy,method->name);
    gchar* methodName = strstr(methodNameCopy, "__");
    if (methodName == NULL) {
        g_string_append_printf(resGString, CONTENT_BETWEEN_METHODNAME_EMPTY, object_path, object_path, method->name);
        g_debug("converting_answer_to_strings() \nCONTENTS_OF_PARENTHESES: %s\n", resGString->str);
    } else {
        guint position = methodName - methodNameCopy; //get delim position
        gchar* actionName = malloc(position + 1);
        strncpy(actionName, methodNameCopy, position);
        actionName[position] = '\000';
        methodName += 2; //remove "__"
        g_string_append_printf(resGString, CONTENT_BETWEEN_METHODNAME, object_path, methodName, object_path, methodName);
        g_debug("converting_answer_to_strings() \nCONTENTS_OF_PARENTHESES: %s\n", resGString->str);
        free(actionName);
    }

    regex = g_regex_new (resGString->str, (GRegexCompileFlags)0, (GRegexMatchFlags)0, &error);
    g_regex_match (regex, request, (GRegexMatchFlags)0, &matchInfo);

    while (g_match_info_matches (matchInfo)) {
        gchar *result = g_match_info_fetch (matchInfo, 0);
        g_debug("converting_answer_to_strings() %s\n", result);
        list = g_list_append(list, result);
        g_match_info_next (matchInfo, &error);

        g_debug("result %s\n",result);
    }
    g_string_free(resGString, TRUE);
    g_debug("converting_answer_to_strings()\nparsing_answer_start  glist: %d\n",g_list_length (list));
    g_info("converting_answer_to_strings()");
    return list;

}
