/***********************************************************************************************************************
**
** Copyright (C) 2021 BaseALT Ltd.
** Authors: Valery Sinelnikov <greh@basealt.ru>, Ivan Savin <svn17@basealt.ru>
**
** This program is free software; you can redistribute it and/or
** modify it under the terms of the GNU General Public License
** as published by the Free Software Foundation; either version 2
** of the License, or (at your option) any later version.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software
** Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
**
***********************************************************************************************************************/
#include "polkitcheck.h"
static guint32
get_sender_data(GDBusConnection* connection, const gchar *sender, const gchar *method);

static void
replace_char(gchar* text, gchar oldChar, gchar newChar)
{
    for(int i = 0; text[i] && (i < 100); i++){
        if(text[i] == oldChar){
            text[i] = newChar;
        }
    }
}
gboolean
authorizationcheck(GDBusConnection* connection, const gchar *sender, gchar *action_id, const gchar *object_path)
{
    PolkitSubject *subject;
    PolkitAuthority *authority;
    GCancellable *cancellable;
    gboolean res = FALSE;
    replace_char(action_id, '/', '.');
    pid_t pid_sender = (pid_t) get_sender_data (connection, sender, "GetConnectionUnixProcessID");
    gint uid_sender = (gint) get_sender_data (connection, sender, "GetConnectionUnixUser");

    authority = polkit_authority_get_sync (NULL, NULL);
    subject = polkit_unix_process_new_for_owner (pid_sender, 0, uid_sender);
    cancellable = g_cancellable_new ();


    PolkitAuthorizationResult * result = polkit_authority_check_authorization_sync
            (authority,
             subject,
             action_id,
             NULL,
             POLKIT_CHECK_AUTHORIZATION_FLAGS_ALLOW_USER_INTERACTION,
             NULL,
             NULL);
    GString *module_write_GString = g_string_new("");
    GString *module_read_GString = g_string_new("");
    g_string_printf(module_write_GString,"%s/write" , (object_path+1));
    g_string_printf(module_read_GString,"%s/write" , (object_path+1));
//    g_string_replace(module_write_GString,"/", ".", 0);
//    g_string_replace(module_read_GString,"/", ".", 0);
    replace_char(module_write_GString->str,'/', '.');
    replace_char(module_read_GString->str,'/', '.');
    g_info("authorizationcheck()\naction_id for module: \nwrite = %s \nread = %s \n",
           module_write_GString->str, module_read_GString->str);

    if(!result){
        if (altsearch("write", action_id)){
            result = polkit_authority_check_authorization_sync
                    (authority,
                     subject,
                     module_write_GString->str,
                     NULL,
                     POLKIT_CHECK_AUTHORIZATION_FLAGS_ALLOW_USER_INTERACTION,
                     NULL,
                     NULL);
        }
        else {
            result = polkit_authority_check_authorization_sync
                    (authority,
                     subject,
                     module_read_GString->str,
                     NULL,
                     POLKIT_CHECK_AUTHORIZATION_FLAGS_ALLOW_USER_INTERACTION,
                     NULL,
                     NULL);
        }
    }

    if (!result){
        if (altsearch("write", action_id)){
            result = polkit_authority_check_authorization_sync
                    (authority,
                     subject,
                     "ru.basealt.alterator.write",
                     NULL,
                     POLKIT_CHECK_AUTHORIZATION_FLAGS_ALLOW_USER_INTERACTION,
                     NULL,
                     NULL);
        }
        else {
            result = polkit_authority_check_authorization_sync
                    (authority,
                     subject,
                     "ru.basealt.alterator.read",
                     NULL,
                     POLKIT_CHECK_AUTHORIZATION_FLAGS_ALLOW_USER_INTERACTION,
                     NULL,
                     NULL);
        }
    }

    if(result){
        res = polkit_authorization_result_get_is_authorized(result);
        g_object_unref(result);
    }

    g_object_unref (authority);
    g_object_unref (subject);
    g_object_unref (cancellable);
    g_string_free(module_write_GString, TRUE);
    g_string_free(module_read_GString, TRUE);
    g_info("authorizationcheck()");
    return res;
}

static guint32
get_sender_data(GDBusConnection* connection, const gchar *sender, const gchar *method){
    GError *error = NULL;
    GDBusMessage *BusMessage = g_dbus_message_new_method_call("org.freedesktop.DBus",
                                                              "/org/freedesktop/DBus",
                                                              "org.freedesktop.DBus",
                                                              method);
    if(BusMessage == NULL){
        g_debug("get_sender_data()\ng_dbus_message_new_method_call failed \n");
    }
    GVariant* send = g_variant_new("(s)", sender);
    g_dbus_message_set_body (BusMessage, send);
    GDBusMessage* reply_message = g_dbus_connection_send_message_with_reply_sync
            (connection,
             BusMessage,
             G_DBUS_SEND_MESSAGE_FLAGS_NONE,
             -1,
             NULL,
             NULL,
             &error);
    if(reply_message == NULL){
        g_debug("get_sender_data()\ng_dbus_connection_send_message_with_reply_sync failed\n");
    }
    if(g_dbus_message_get_message_type(reply_message) == G_DBUS_MESSAGE_TYPE_ERROR){
        g_dbus_message_to_gerror(reply_message, &error);
        g_debug("get_sender_data()\nError invoking g_dbus_connection_send_message_with_reply_sync: %s\n", error->message);
        g_error_free(error);
    }
    GVariant* var_child = g_variant_get_child_value(g_dbus_message_get_body(reply_message), 0);
    guint32 data = g_variant_get_uint32(var_child);
    g_variant_unref (var_child)  ;
    g_object_unref(BusMessage);
    g_object_unref(reply_message);
    g_info("get_sender_data()");
    return data;
}

gboolean
altsearch(gchar *word,const gchar *content){

    GError *err = NULL;
    GMatchInfo *matchInfo;
    GRegex *regex;
    gboolean found = FALSE;
    regex = g_regex_new (word, (GRegexCompileFlags)0, (GRegexMatchFlags)0, &err);
    g_regex_match (regex, content, (GRegexMatchFlags)0, &matchInfo);

    while (g_match_info_matches (matchInfo)) {
        gchar *result = g_match_info_fetch (matchInfo, 0);
        if(result) found = TRUE;
        g_match_info_next (matchInfo, &err);
        g_free (result);
    }
    g_info("altsearch()");
    return found;
}
