/***********************************************************************************************************************
**
** Copyright (C) 2021 BaseALT Ltd.
** Authors: Valery Sinelnikov <greh@basealt.ru>, Ivan Savin <svn17@basealt.ru>
**
** This program is free software; you can redistribute it and/or
** modify it under the terms of the GNU General Public License
** as published by the Free Software Foundation; either version 2
** of the License, or (at your option) any later version.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software
** Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
**
***********************************************************************************************************************/
#include "requesttowoobus.h"

static gchar*
make_argument_for_checklist (GVariant *args_container) {
    GVariantIter iter;
    GVariant *child;
    const gchar *arg_content;
    gchar *result = NULL;

    g_variant_iter_init (&iter, args_container);
    while ((child = g_variant_iter_next_value (&iter))) {
        g_variant_get(child, g_variant_get_type_string (child), &arg_content);
        if (result == NULL) {
            result = g_strconcat("", arg_content, NULL);
        } else if (!g_strcmp0("", result)) {
            g_free(result);
            result = g_strconcat("", arg_content, NULL);
        } else {
            gchar *tmp = g_strconcat(result, ";", arg_content, NULL);
            g_free(result);
            result = tmp;
        }
        g_variant_unref (child);
    }
    g_info("make_argument_for_checklist()");
    return result;
}

static gchar*
make_arguments_string (GVariant *args_container, GDBusArgInfo **in_args) {
    GVariantIter iter;
    GVariant *child;
    gchar *arg_content;
    GString *resultGString = g_string_new("");

    g_info("make_arguments_string() \ncontainer: %s #####\n", g_variant_get_type_string (args_container));

    g_variant_iter_init (&iter, args_container);
    while ((child = g_variant_iter_next_value (&iter)) && (*in_args)) {
        //signature validation
        if (g_strcmp0(g_variant_get_type_string(child), (*in_args)->signature)) {
            g_warning("make_arguments_string(): the signature is not correct.");
            in_args++;
            g_variant_unref (child);
            continue;
        }
        if (!g_strcmp0((*in_args)->signature, "as") &&
                g_str_has_prefix((*in_args)->name, CHECKLIST_PREFIX)) {
            arg_content = make_argument_for_checklist(child);
            if (arg_content != NULL) {
                gchar* name = (*in_args)->name;
                for (gint i = 0; i < 50 && *name != '#'; i++, name++);//50 is insurance
                name++; //skipping '#'
                g_string_printf(resultGString, "%s %s \"%s\"", resultGString->str, name, arg_content);
                g_free(arg_content);
            }

            in_args++;
            g_variant_unref (child);
            continue;
        } else if (!g_strcmp0((*in_args)->signature, "s")) {
            g_variant_get(child, g_variant_get_type_string (child), &arg_content);
            g_string_append_printf(resultGString," %s \"%s\"", (*in_args)->name, arg_content);
        }

        in_args++;
        g_variant_unref (child);
    }
    g_info("make_arguments_string ()result = %s\n", resultGString->str);
    gchar* result = g_string_free(resultGString, FALSE);
    if (g_strcmp0(result, "")) {
        g_debug("make_arguments_string() result = %s\n", result);
        return result;
    } else {
        g_debug("make_arguments_string() result = NULL\n");
        return NULL;
    }
    g_info("make_arguments_string()");
}

gchar*
make_request_woobus(const gchar* object_path, const GDBusMethodInfo* method, GVariant *parameters) {
    GString *requestGString = g_string_new("");
    gchar* in_args_str = NULL;
    gchar* action = NULL;
    if (method == NULL) {
        g_warning("make_request_woobus() Not correct method");
        return NULL;
    }

    gchar methodNameCopy[strlen(method->name)];
    strcpy(methodNameCopy,method->name);
    gchar* methodName = strstr(methodNameCopy, "__");
    if (methodName == NULL) {
        g_debug("make_request_woobus()\nmethodNameCopy : %s\n", methodNameCopy);
        action = methodNameCopy;
        g_string_append_printf(requestGString, "(\"%s\" action \"%s\"", object_path, method->name);
        g_debug("make_request_woobus()\nmethodName == NULL: %s\n", requestGString->str);
        // return;
    } else {
        guint position = (guint)(methodName - methodNameCopy); //get delim position
        g_debug("make_request_woobus()\n__methodName : %s\n", methodName);
        g_debug("make_request_woobus()\nposition : %d\n", position);
        gchar* actionName = malloc(position + 1); // + terminated null
        g_debug("make_request_woobus()\nmethodNameCopy : %s\n", methodNameCopy);
        strncpy(actionName, methodNameCopy, position);
        actionName[position] = '\000';
        g_debug("make_request_woobus()\nactionName : %s\n", actionName);
        methodName += 2; //remove "__"
        g_debug("make_request_woobus()\nmethodName : %s\n", methodName);
        g_debug("make_request_woobus()\nmethodNameCopy : %s\n", methodNameCopy);
        g_string_append_printf(requestGString,"(\"%s/%s\" action \"%s\"", object_path, methodName, actionName);
        free(actionName);
    }
    GDBusArgInfo **ins = method->in_args;
    in_args_str = make_arguments_string(parameters, ins);
    if (in_args_str != NULL) {
        g_string_append_printf(requestGString, " %s)", in_args_str);
        free(in_args_str);
    } else {
        g_string_append_printf(requestGString,"%s", ")");
    }
    g_message("request: %s \n", requestGString->str);
    g_info("make_request_woobus()");
    gchar* request = g_string_free(requestGString, FALSE);
    return request;
}
